class CharsDelete:
    def __init__(self, chars):
        self.chars = chars

    def __call__(self, data: str) -> str:
        for char in self.chars:
            data = data.replace(char, "")

        return data



import re


class PatternReplace:
    def __init__(self, pattern: str, target: str):
        self.pattern = pattern
        self.target = target

        self.p = re.compile(self.pattern)

    def __call__(self, data: str) -> str:
        return self.p.sub(self.target, data)



import logging
import os
import sys
from urllib.request import urlretrieve

if sys.version_info >= (3,):
    import urllib.request as urllib2
    import urllib.parse as urlparse
    from urllib.request import urlretrieve
else:
    import urllib2
    import urlparse
    from urllib import urlretrieve


def getEmbeddings(name):
    if not os.path.isfile(name):
        download("https://public.ukp.informatik.tu-darmstadt.de/reimers/embeddings/"+name)


def download(url, destination=os.curdir, silent=False):
    filename = os.path.basename(urlparse.urlparse(url).path) or 'downloaded.file'

    def get_size():
        meta = urllib2.urlopen(url).info()
        meta_func = meta.getheaders if hasattr(
            meta, 'getheaders') else meta.get_all
        meta_length = meta_func('Content-Length')
        try:
            return int(meta_length[0])
        except:
            return 0

    def kb_to_mb(kb):
        return kb / 1024.0 / 1024.0

    def callback(blocks, block_size, total_size):
        current = blocks * block_size
        percent = 100.0 * current / total_size
        line = '[{0}{1}]'.format(
            '=' * int(percent / 2), ' ' * (50 - int(percent / 2)))
        status = '\r{0:3.0f}%{1} {2:3.1f}/{3:3.1f} MB'
        sys.stdout.write(
            status.format(
                percent, line, kb_to_mb(current), kb_to_mb(total_size)))

    path = os.path.join(destination, filename)

    logging.info(
        'Downloading: {0} ({1:3.1f} MB)'.format(url, kb_to_mb(get_size())))
    try:
        (path, headers) = urlretrieve(url, path, None if silent else callback)
    except:
        os.remove(path)
        raise Exception("Can't download {0}".format(path))
    else:
        print()
        logging.info('Downloaded to: {0}'.format(path))

    return path


from typing import Union, List


class CharsReplace:
    def __init__(self, source: Union[list, str], target: Union[list, str]):
        if isinstance(source, str):
            source = list(source)
        if isinstance(target, str):
            target = list(target)

        assert len(source) == len(target)

        self.source = source
        self.target = target

        self.mapping = dict(zip(self.source, self.target))

    def __call__(self, data: list) -> list:
        return self.batch_process(data)

    def batch_process(self, data: List[List[str]]):
        return [self.process(i) for i in data]

    def process(self, data: list) -> list:
        return [self.mapping.get(i, i) for i in data]



import numpy as np
import tensorflow as tf


def export_numpy_to_checkpoint(data, checkpoint, tensor_name="embedding"):
    variable = tf.Variable(data, dtype=tf.float32, trainable=False, name=tensor_name)

    # Add an op to initialize the variables.
    init_op = tf.global_variables_initializer()

    # Add ops to save and restore all the variables.
    saver = tf.train.Saver()

    with tf.Session() as sess:
        # sess = tf_debug.LocalCLIDebugWrapperSession(sess)

        sess.run(init_op)
        result = sess.run(variable)

        print(result)

        save_path = saver.save(sess, checkpoint)
        print("Model saved in path: %s" % save_path)



import re


def wordNormalize(word):
    word = word.lower()
    word = word.replace("--", "-")
    word = re.sub("\"+", '"', word)
    word = re.sub("[0-9]{4}-[0-9]{2}-[0-9]{2}", 'DATE_TOKEN', word)
    word = re.sub("[0-9]{2}:[0-9]{2}:[0-9]{2}", 'TIME_TOKEN', word)
    word = re.sub("[0-9]{2}:[0-9]{2}", 'TIME_TOKEN', word)
    word = re.sub("[0-9.,]+", 'NUMBER_TOKEN', word)
    return word


import gzip
import logging
import os

import nltk
import numpy as np

from nlp_utils.dataset.download import getEmbeddings
from nlp_utils.embedding.word_embedding import wordNormalize


def readEmbeddings(embeddingsPath, datasetFiles=tuple(), frequencyThresholdUnknownTokens=None, reducePretrainedEmbeddings=False):
    """
    Reads the embeddingsPath.
    :param embeddingsPath: File path to pretrained embeddings
    :param datasetName:
    :param datasetFiles:
    :param frequencyThresholdUnknownTokens:
    :param reducePretrainedEmbeddings:
    :return:
    """
    # Check that the embeddings file exists
    if not os.path.isfile(embeddingsPath):
        if embeddingsPath in ['komninos_english_embeddings.gz', 'levy_english_dependency_embeddings.gz', 'reimers_german_embeddings.gz']:
            getEmbeddings(embeddingsPath)
        else:
            print("The embeddings file %s was not found" % embeddingsPath)
            exit()

    logging.info("Generate new embeddings files for a dataset")

    neededVocab = {}
    if reducePretrainedEmbeddings:
        logging.info("Compute which tokens are required for the experiment")

        def createDict(filename, tokenPos, vocab):
            for line in open(filename):
                if line.startswith('#'):
                    continue
                splits = line.strip().split()
                if len(splits) > 1:
                    word = splits[tokenPos]
                    wordLower = word.lower()
                    wordNormalized = wordNormalize(wordLower)

                    vocab[word] = True
                    vocab[wordLower] = True
                    vocab[wordNormalized] = True

        for dataset in datasetFiles:
            dataColumnsIdx = {y: x for x, y in dataset['cols'].items()}
            tokenIdx = dataColumnsIdx['tokens']
            datasetPath = 'data/%s/' % dataset['name']

            for dataset in ['train.txt', 'dev.txt', 'test.txt']:
                createDict(datasetPath + dataset, tokenIdx, neededVocab)

    # :: Read in word embeddings ::
    logging.info("Read file: %s" % embeddingsPath)
    word2Idx = {}
    embeddings = []

    embeddingsIn = gzip.open(embeddingsPath, "rt") if embeddingsPath.endswith('.gz') else open(embeddingsPath,
                                                                                               encoding="utf8")

    embeddingsDimension = None

    for line in embeddingsIn:
        split = line.rstrip().split(" ")
        word = split[0]

        if embeddingsDimension == None:
            embeddingsDimension = len(split) - 1

        if (len(
                split) - 1) != embeddingsDimension:  # Assure that all lines in the embeddings file are of the same length
            print("ERROR: A line in the embeddings file had more or less  dimensions than expected. Skip token.")
            continue

        if len(word2Idx) == 0:  # Add padding+unknown
            word2Idx["PADDING_TOKEN"] = len(word2Idx)
            vector = np.zeros(embeddingsDimension)
            embeddings.append(vector)

            word2Idx["UNKNOWN_TOKEN"] = len(word2Idx)
            vector = np.random.uniform(-0.25, 0.25, embeddingsDimension)  # Alternativ -sqrt(3/dim) ... sqrt(3/dim)
            embeddings.append(vector)

        vector = np.array([float(num) for num in split[1:]])

        if len(neededVocab) == 0 or word in neededVocab:
            if word not in word2Idx:
                embeddings.append(vector)
                word2Idx[word] = len(word2Idx)

    # Extend embeddings file with new tokens
    def createFD(filename, tokenIndex, fd, word2Idx):
        for line in open(filename):
            if line.startswith('#'):
                continue

            splits = line.strip().split()

            if len(splits) > 1:
                word = splits[tokenIndex]
                wordLower = word.lower()
                wordNormalized = wordNormalize(wordLower)

                if word not in word2Idx and wordLower not in word2Idx and wordNormalized not in word2Idx:
                    fd[wordNormalized] += 1

    if frequencyThresholdUnknownTokens != None and frequencyThresholdUnknownTokens >= 0:
        fd = nltk.FreqDist()
        for datasetName, datasetFile in datasetFiles.items():
            dataColumnsIdx = {y: x for x, y in datasetFile['columns'].items()}
            tokenIdx = dataColumnsIdx['tokens']
            datasetPath = 'data/%s/' % datasetName
            createFD(datasetPath + 'train.txt', tokenIdx, fd, word2Idx)

        addedWords = 0
        for word, freq in fd.most_common(10000):
            if freq < frequencyThresholdUnknownTokens:
                break

            addedWords += 1
            word2Idx[word] = len(word2Idx)
            vector = np.random.uniform(-0.25, 0.25, len(split) - 1)  # Alternativ -sqrt(3/dim) ... sqrt(3/dim)
            embeddings.append(vector)

            assert (len(word2Idx) == len(embeddings))

        logging.info("Added words: %d" % addedWords)
    embeddings = np.array(embeddings)

    return embeddings, word2Idx


from setuptools import setup, find_packages

setup(
    name="nlp_utils",
    version="0.5.0",
    packages=find_packages(exclude=["tests.*"]),
    url="",
    license="MIT",
    author="Xiaoquan Kong",
    author_email="u1mail2me@gmail.com",
    description="Utils for NLP",
    install_requires=["nltk", "numpy", "tensorflow", "micro_toolkit"],
)



import collections
from typing import Union, List, Tuple, Hashable, Any

from nlp_utils.preprocess import lookup_table

OOV = collections.namedtuple("OOV", ["value", "id"])
Padding = collections.namedtuple("Padding", ["value", "id"])


class VocabularyLookupTable(lookup_table.LookupTable):
    def __init__(self, dictionary, oov_id, oov_value):
        super().__init__(dictionary, oov=oov_id)

        self.oov_id = oov_id
        self.oov_value = oov_value

    def _do_inverse_lookup(self, key: Hashable) -> Any:
        try:
            return self.inverse_table[key]
        except KeyError:
            if key == self.oov_id:
                return self.oov_value

            raise


class Vocabulary:
    def __init__(self, dictionary: list, oov: str = None, padding: str = None):
        """oov must in dictionary, padding must in dictionary"""

        if len(dictionary) != len(set(dictionary)):
            raise ValueError("dictionary contains duplicated value")

        self.dictionary = dictionary

        # assign oov
        self.oov = None
        if oov is not None:
            self._set_oov(oov)

        # assign padding
        self.padding = None
        if padding is not None:
            self._set_padding(padding)

        self.table = VocabularyLookupTable(
            dictionary,
            oov_id=self.oov.id if self.oov else None,
            oov_value=self.oov.value if self.oov else None,
        )

    def _set_oov(self, value: str):
        # check
        assert value in self.dictionary

        oov_id = self.dictionary.index(value)
        self.oov = OOV(value, oov_id)

    def _set_padding(self, value: str):
        # check
        assert value in self.dictionary

        padding_id = self.dictionary.index(value)
        self.padding = Padding(value, padding_id)

    def lookup(self, items):
        return self.table.batch_lookup(items)

    def inverse_lookup(self, items):
        return self.table.batch_inverse_lookup(items)

    def get_config(self) -> dict:
        return {
            "dictionary": self.dictionary,
            "oov": self.oov.value if self.oov else None,
            "padding": self.padding.value if self.padding else None,
        }

    @classmethod
    def from_config(cls, config) -> "Vocabulary":
        return cls(**config)

    def size(self) -> int:
        return len(self.dictionary)

    def dictionary_to_list(self) -> list:
        return self.dictionary



import json

import pkg_resources
from nlp_utils.preprocess.clean.chars_replace import CharsReplace
from nlp_utils.preprocess.lookup_table import LookupTable
from nlp_utils.preprocess.padding import SequencePaddding


class TextPreprocess:
    def __init__(self, pipeline: list):
        self.pipeline = pipeline

    def __call__(self, data):
        for processor in self.pipeline:
            data = processor(data)

        return data

    def vocab_size(self):
        lookup_table = None
        for p in self.pipeline:
            if isinstance(p, LookupTable):
                lookup_table = p
                break

        return lookup_table.size() if lookup_table else None


def fixed_len_preprocess(max_length=45):
    data_file = pkg_resources.resource_filename(
        __name__, "../resources/gb2312_dict.json"
    )
    with open(data_file) as fd:
        data = json.load(fd)

    padding = "<PAD>"
    oov_key = "<UNK>"
    lookup_table = [padding] + [oov_key] + data["words"]

    pre = data["preprocessing"]
    char_replace = list(
        zip(
            (i["source"], i["target"]) for i in [pre[2], pre[3], pre[4], pre[5], pre[6]]
        )
    )

    char_replace_src = char_replace[0]
    char_replace_target = char_replace[1]

    pipeline = [
        CharsReplace(char_replace_src, char_replace_target),
        SequencePaddding(padding, max_length),
        LookupTable(lookup_table, oov_key=oov_key),
    ]
    tp = TextPreprocess(pipeline)
    return tp


if __name__ == "__main__":
    data = "some text"
    tp = fixed_len_preprocess()
    reuslt = tp(data)



import json
from typing import Union, List, Tuple, Hashable, Any


class LookupTable(object):
    """Bidirectional Lookup Table with out-of-vocabulary (OOV) supporting.

    this class support `lookup()` and `inverse_lookup()` with OOV supporting

    Attributes:
        table: A dict contains mapping for `lookup()` to query.
        inverse_table: A dict contains mapping for `inverse_lookup` to query.
        oov: A hashable object as return value for `lookup()` when OOV
             if it's not None.
        oov_key: A hashable object act as query key for `lookup()` when OOV
            if it's not None and `oov` is None.
        inverse_oov: A hashable object as return value for `inverse_lookup()`
            when OOV if it's not None.
        inverse_oov_key: A hashable object act as query key for
            `inverse_lookup()` if it's not None and `inverse_oov` is None.
    """

    def __init__(
        self,
        table: Union[dict, list, tuple],
        oov=None,
        oov_key=None,
        inverse_oov=None,
        inverse_oov_key=None,
    ):
        """
        Args:
            table: A dictionary represented as dict or sequence, list/tuple will
                convert to dict by using index as key, `lookup()` will mapping
                it's key to value, `inverse_lookup()` will mapping it's value to
                key.
            oov: A hashable object (typically a int) or None, if it is not a
                None, will use this as return value in `lookup()`, when the
                query out-of-vocabulary (OOV).
            oov_key: A hashable object (typically a str) or None, if it is not
                None, will use this as key to query value in `lookup()` when OOV,
                this arg will be used if and only if `oov` is set to None.
            inverse_oov: A hashable object (typically a str) or None, if it is
                not None, will use this as return value in `inverse_lookup()`
                when OOV.
            inverse_oov_key: A hashable object (typically an int) or None, if
                it's not None, will use this as key to query value in
                `inverse_lookup()` when OOV, this arg will be used if and only if
                `inverse_oov` is set to None.

        Raises:
            ValueError: An argument checking error occurred.
        """

        if isinstance(table, (list, tuple)):
            table = {v: k for k, v in enumerate(table)}
        if not isinstance(table, dict):
            raise ValueError(
                "table argument do not accept type: {}".format(type(table))
            )

        if oov is not None and oov_key is not None:
            raise ValueError("Only one of oov and oov_key can be set to non None")
        if inverse_oov is not None and inverse_oov_key is not None:
            raise ValueError(
                "Only one of inverse_oov and inverse_oov_key can be set to non None"
            )

        self.table = table
        self.inverse_table = {v: k for k, v in table.items()}
        if len(self.table) != len(self.inverse_table):
            raise ValueError(
                "table values are duplicated, reverse table cannot be created"
            )

        self.oov = oov
        self.oov_key = oov_key
        self.inverse_oov = inverse_oov
        self.inverse_oov_key = inverse_oov_key

        if self.oov is not None and self.oov not in self.table.values():
            print("WARNING: oov not in table")
        if self.oov_key is not None and self.oov_key not in self.table.keys():
            raise ValueError("WARNING: oov_key not in table")
        if (
            self.inverse_oov is not None
            and self.inverse_oov not in self.inverse_table.values()
        ):
            print("WARNING: inverse_oov not in inverse_table")
        if (
            self.inverse_oov_key is not None
            and self.inverse_oov_key not in self.inverse_table.keys()
        ):
            raise ValueError("WARNING: inverse_oov_key not in inverse_table")

    def __call__(self, key):
        return self.batch_lookup(key)

    def batch_lookup(self, key):
        """
        Raises:
            KeyError:  when `ovv` and `oov_key` is not set, unhandleable OOV occurs.
        """

        return [self.lookup(i) for i in key]

    def lookup(
        self, key: Union[Hashable, List[Hashable], Tuple[Hashable]]
    ) -> Union[Any, List[Any]]:
        """
        Raises:
            KeyError:  when `ovv` and `oov_key` is not set, unhandleable OOV occurs.
        """

        if isinstance(key, (list, tuple)):
            return tuple(self._do_lookup(i) for i in key)
        return self._do_lookup(key)

    def _do_lookup(self, key: Hashable) -> Any:
        try:
            return self.table[key]
        except KeyError:
            # TODO(Xiaoquan Kong): Add callbacks when OOV occurs
            if self.oov is not None:
                return self.oov
            elif self.oov_key is not None:
                return self.table[self.oov_key]
            else:
                raise

    def batch_inverse_lookup(self, key):
        """
        Raises:
            KeyError:  when `ovv` and `oov_key` is not set, unhandleable OOV occurs.
        """

        return [self.inverse_lookup(i) for i in key]

    def inverse_lookup(
        self, key: Union[Hashable, List[Hashable], Tuple[Hashable]]
    ) -> Union[Any, List[Any]]:
        """
        Raises:
            KeyError:  when `ovv` and `oov_key` is not set, unhandleable OOV occurs.
        """

        if isinstance(key, (list, tuple)):
            return tuple(self._do_inverse_lookup(i) for i in key)
        return self._do_inverse_lookup(key)

    def _do_inverse_lookup(self, key: Hashable) -> Any:
        try:
            return self.inverse_table[key]
        except KeyError:
            # TODO(Xiaoquan Kong): Add callbacks when OOV occurs
            if self.inverse_oov is not None:
                return self.inverse_oov
            elif self.inverse_oov_key is not None:
                return self.inverse_table[self.inverse_oov_key]
            else:
                raise

    def size(self) -> int:
        return len(self.table)

    @classmethod
    def read_from_file(cls, data_file, kwargs: dict = {}):
        with open(data_file, "rt") as fd:
            paired_dict = json.load(fd)

            return cls(paired_dict, **kwargs)

    # alias for compatible
    load_from_file = read_from_file

    def write_to_file(self, data_file):
        with open(data_file, "wt") as fd:
            # set ensure_ascii=False for human readability of dumped file
            json.dump(self.table, fd, ensure_ascii=False)

    def dump_to_file(self, data_file):
        # alias for compatible
        return self.write_to_file(data_file)

    def get_config(self):
        return {
            "oov": self.oov,
            "oov_key": self.oov_key,
            "inverse_oov": self.inverse_oov,
            "inverse_oov_key": self.inverse_oov_key,
        }
